@@ -2,23 +2,25 @@ use crate::args::ArgSpec;
 use crate::{arg, arg_enum, args};
 use cdx::expr;
 use cdx::matcher::*;
-use cdx::{get_writer, Error, HeaderChecker, HeaderMode, LookbackReader, Result, HEADER_MODE};
+use cdx::util::{get_writer, HeaderChecker, HeaderMode, LookbackReader, Result, HEADER_MODE};
 use std::str::FromStr;
 
 pub fn main(argv: &[String]) -> Result<()> {
     let prog = args::ProgSpec::new("Select uniq lines.", args::FileCount::Many);
-    const A: [ArgSpec; 6] = [
+    const A: [ArgSpec; 7] = [
         arg! {"pattern", "p", "Col,Spec,Pattern", "Select line where this col matches this pattern."},
         arg! {"show-matchers", "s", "", "Print available matchers"},
         arg! {"show-const", "", "", "Print available constants"},
         arg! {"show-func", "", "", "Print available functions"},
         arg! {"or", "o", "", "A line matches if any of the matchers matches."},
+        arg! {"invert", "v", "", "Print lines that don't match."},
         arg_enum! {"header", "h", "Mode", "header requirements", &HEADER_MODE},
     ];
     let (args, files) = args::parse(&prog, &A, argv);
 
     let mut checker = HeaderChecker::new();
     let mut list = MultiLineMatcher::new(MultiMode::And);
+    let mut reverse = false;
     for x in args {
         if x.name == "header" {
             checker.mode = HeaderMode::from_str(&x.value)?;
@@ -29,6 +31,8 @@ pub fn main(argv: &[String]) -> Result<()> {
             return Ok(());
         } else if x.name == "or" {
             list.multi = MultiMode::Or;
+        } else if x.name == "invert" {
+            reverse = true;
         } else if x.name == "show-const" {
             expr::show_const();
             return Ok(());
@@ -42,9 +46,6 @@ pub fn main(argv: &[String]) -> Result<()> {
 
     let mut w = get_writer("-")?;
 
-    let grep_mode = true;
-    let reverse = false;
-    let max_fails = 5;
     let mut first_file = true;
 
     for x in &files {
@@ -63,33 +64,15 @@ pub fn main(argv: &[String]) -> Result<()> {
         if f.is_done() {
             continue;
         }
-        if grep_mode {
-            loop {
-                if list.ok(f.curr_line()) ^ reverse {
-                    // write previous lines of context if necessary
-                    f.write_curr(&mut w)?;
-                } else {
-                    // write more lines of context if necessary
-                }
-                if f.getline()? {
-                    break;
-                }
+        loop {
+            if list.ok(f.curr_line()) ^ reverse {
+                // write previous lines of context if necessary
+                f.write_curr(&mut w)?;
+            } else {
+                // write more lines of context if necessary
             }
-        } else {
-            let mut fails = 0;
-            loop {
-                if !list.ok_verbose(f.curr_line(), false) {
-                    fails += 1;
-                    if fails >= max_fails {
-                        break;
-                    }
-                }
-                if f.getline()? {
-                    break;
-                }
-            }
-            if fails > 0 {
-                return Err(Error::Silent);
+            if f.getline()? {
+                break;
             }
         }
     }
