@@ -1,12 +1,10 @@
 //! Handles conversion between named column sets and lists of column numbers
 //! Also helps with selecting those columns from a line of text
 
-use crate::num::str_to_u_whole;
-use crate::text::{Case, Text};
-use crate::{err, Error, Result, StringLine, TextLine};
-use lazy_static::lazy_static;
-use regex::Regex;
-use std::cmp::Ordering;
+use crate::expr::Expr;
+use crate::matcher::MatchMaker;
+use crate::text::Text;
+use crate::util::{err, find_close, Error, Result, StringLine, TextLine};
 use std::collections::HashSet;
 use std::io::Write;
 use std::{fmt, str};
@@ -55,8 +53,8 @@ pub struct ColumnHeader {
 }
 
 /// is 'name' a valid column name
-pub const fn is_valid_column_name(name: &str) -> bool {
-    !name.is_empty()
+pub fn is_valid_column_name(name: &str) -> bool {
+    get_col_name(name) == name.len()
 }
 
 /// throw an error is name is not a valid column name
@@ -210,7 +208,7 @@ impl ColumnHeader {
 ///    assert_eq!(s.get_cols_num(), &[0,4]);
 ///
 ///    let mut s = ColumnSet::new();
-///    s.add_yes(">s<=two");
+///    s.add_yes("(range,>s<=two)");
 ///    s.lookup(&header);
 ///    assert_eq!(s.get_cols_num(), &[2,3]);
 /// ```
@@ -257,10 +255,10 @@ pub struct ScopedValue {
 
 impl ScopedValue {
     /// one string
-    pub fn new(spec: &str, del: char) -> Self {
+    pub fn new(spec: &str, del: char) -> Result<Self> {
         let mut s = Self::default();
         if spec.is_empty() {
-            return s;
+            return Ok(s);
         }
         let mut spec = spec;
         let ch = spec.first();
@@ -268,7 +266,7 @@ impl ScopedValue {
             spec.skip_first();
             if spec.is_empty() {
                 s.value.push(del);
-                return s;
+                return Ok(s);
             }
             let ch = spec.first();
             if ch == del {
@@ -278,19 +276,19 @@ impl ScopedValue {
         while !spec.is_empty() {
             let ch = spec.take_first();
             if ch == del {
-                s.cols.add_yes(spec);
-                return s;
+                s.cols.add_yes(spec)?;
+                return Ok(s);
             }
             s.value.push(ch);
         }
-        s
+        Ok(s)
     }
     /// two strings
-    pub fn new2(value: &str, cols: &str) -> Self {
+    pub fn new2(value: &str, cols: &str) -> Result<Self> {
         let mut s = Self::default();
-        s.cols.add_yes(cols);
+        s.cols.add_yes(cols)?;
         s.value = value.to_string();
-        s
+        Ok(s)
     }
     /// resolve named columns
     pub fn lookup(&mut self, fieldnames: &[&str]) -> Result<()> {
@@ -352,12 +350,14 @@ impl ScopedValues {
     }
 
     /// add one ScopedValue
-    pub fn add(&mut self, spec: &str, del: char) {
-        self.data.push(ScopedValue::new(spec, del));
+    pub fn add(&mut self, spec: &str, del: char) -> Result<()> {
+        self.data.push(ScopedValue::new(spec, del)?);
+        Ok(())
     }
     /// add one ScopedValue
-    pub fn add2(&mut self, value: &str, cols: &str) {
-        self.data.push(ScopedValue::new2(value, cols));
+    pub fn add2(&mut self, value: &str, cols: &str) -> Result<()> {
+        self.data.push(ScopedValue::new2(value, cols)?);
+        Ok(())
     }
     /// get appropriate value for the column
     pub fn get(&self, col: usize) -> &str {
@@ -407,6 +407,45 @@ pub trait ColumnFun {
     fn lookup(&mut self, fieldnames: &[&str]) -> Result<()>;
 }
 
+/// write a single column
+#[derive(Debug, Default)]
+pub struct ColumnExpr {
+    name: String,
+    expr: Expr,
+}
+
+impl ColumnExpr {
+    /// new from Expr or Name:Expr
+    pub fn new(spec: &str) -> Result<Self> {
+        if let Some((a, b)) = spec.split_once(':') {
+            Ok(Self {
+                name: a.to_string(),
+                expr: Expr::new(b)?,
+            })
+        } else {
+            Ok(Self {
+                name: String::new(),
+                expr: Expr::new(spec)?,
+            })
+        }
+    }
+}
+
+impl ColumnFun for ColumnExpr {
+    fn add_names(&self, w: &mut ColumnHeader, _head: &StringLine) -> Result<()> {
+        w.push(&self.name)
+    }
+    /// write the column values (called many times)
+    fn write(&mut self, w: &mut dyn Write, line: &TextLine, _delim: u8) -> Result<()> {
+        write!(w, "{}", self.expr.eval(line))?;
+        Ok(())
+    }
+    /// resolve any named columns
+    fn lookup(&mut self, fieldnames: &[&str]) -> Result<()> {
+        self.expr.lookup(fieldnames)
+    }
+}
+
 /// write a single column
 #[derive(Debug, Default)]
 pub struct ColumnSingle {
@@ -554,43 +593,63 @@ impl ColumnSet {
     }
 
     /// Create an column set from a spec, e.g. "1-3"
-    pub fn from_spec(spec: &str) -> Self {
+    pub fn from_spec(spec: &str) -> Result<Self> {
         let mut s = Self::default();
-        s.add_yes(spec);
-        s
+        s.add_yes(spec)?;
+        Ok(s)
     }
 
     /// Add some columns to be selected
-    pub fn add_yes(&mut self, spec: &str) {
-        self.add(spec, false);
+    pub fn add_yes(&mut self, spec: &str) -> Result<()> {
+        self.add(spec, false)
     }
 
     /// Add some columns to be rejected.
-    pub fn add_no(&mut self, spec: &str) {
-        self.add(spec, true);
+    pub fn add_no(&mut self, spec: &str) -> Result<()> {
+        self.add(spec, true)
     }
 
-    /// Add a spec, "yes" or "no" based on a flag.
-    pub fn add(&mut self, spec: &str, negate: bool) {
-        for s in spec.split(',') {
-            if let Some(stripped) = s.strip_prefix('~') {
-                let st = stripped.to_string();
-                if negate {
-                    self.pos.push(st);
-                } else {
-                    self.neg.push(st);
-                }
+    /// Add a single range, "yes" or "no" based on a flag.
+    pub fn add_one(&mut self, s: &str, negate: bool) {
+        if let Some(stripped) = s.strip_prefix('~') {
+            let st = stripped.to_string();
+            if negate {
+                self.pos.push(st);
             } else {
-                let st = s.to_string();
-                if negate {
-                    self.neg.push(st);
-                } else {
-                    self.pos.push(st);
-                }
+                self.neg.push(st);
+            }
+        } else {
+            let st = s.to_string();
+            if negate {
+                self.neg.push(st);
+            } else {
+                self.pos.push(st);
             }
         }
     }
 
+    /// Add a list or ranges, "yes" or "no" based on a flag.
+    pub fn add(&mut self, spec: &str, negate: bool) -> Result<()> {
+        let mut spc = spec;
+        loop {
+            if spc.first() == '(' {
+                let pos = find_close(spc)?;
+                self.add_one(&spc[..pos], negate);
+                spc = &spc[pos..];
+                if spc.is_empty() {
+                    break;
+                }
+            } else if let Some((a, b)) = spc.split_once(',') {
+                self.add_one(a, negate);
+                spc = b;
+            } else {
+                self.add_one(spc, negate);
+                break;
+            }
+        }
+        Ok(())
+    }
+
     /// Turn column name into column number
     /// To also handle numbers and ranges, use `lookup1`
     /// # Examples
@@ -628,7 +687,7 @@ impl ColumnSet {
     /// ```
     pub fn single(fieldnames: &[&str], colname: &str) -> Result<usize> {
         if let Some(stripped) = colname.strip_prefix('+') {
-            let n = str_to_u_whole(stripped.as_bytes())? as usize;
+            let n = stripped.to_usize_whole()?;
             let len = fieldnames.len();
             if n > len {
                 err!("Column {} out of bounds", colname)
@@ -636,9 +695,9 @@ impl ColumnSet {
                 Ok(len - n)
             }
         } else {
-            let ch = colname.chars().next().unwrap();
-            if ch.is_ascii_digit() {
-                let n = str_to_u_whole(colname.as_bytes())? as usize;
+            let ch = colname.first();
+            if ch.is_ascii_digit() && ch != '0' {
+                let n = colname.to_usize_whole()?;
                 if n < 1 {
                     err!("Column {} out of bounds", colname)
                 } else {
@@ -650,79 +709,18 @@ impl ColumnSet {
         }
     }
 
-    /// determine if two strings match, given an operator
-    fn do_compare(left: &str, right: &str, operator: &str) -> Result<bool> {
-        let val = left.cmp(right);
-        if operator == "<" {
-            return Ok(val == Ordering::Less);
-        }
-        if operator == "<=" {
-            return Ok(val != Ordering::Greater);
-        }
-        if operator == ">" {
-            return Ok(val == Ordering::Greater);
-        }
-        if operator == ">=" {
-            return Ok(val != Ordering::Less);
-        }
-        err!(operator.to_string())
-    }
-
-    /// determine if a name is within a range
-    fn do_compare2(name: &str, key1: &str, op1: &str, key2: &str, op2: &str) -> Result<bool> {
-        Ok(Self::do_compare(name, key1, op1)? && Self::do_compare(name, key2, op2)?)
-    }
-
-    /// Return all the fields that match the glob
-    fn glob_range(fieldnames: &[&str], rng: &str) -> Result<Vec<usize>> {
+    /// Return all the fields that match the Matcher
+    fn match_range(fieldnames: &[&str], rng: &str) -> Result<Vec<usize>> {
         let mut ret = Vec::new();
+        let c = MatchMaker::make(rng)?;
         for f in fieldnames.iter().enumerate() {
-            if f.1.glob(rng, Case::Sens) {
+            if c.smatch(f.1) {
                 // allow case insensitive?
                 ret.push(f.0);
             }
         }
         Ok(ret)
     }
-    /// Return all the field numbers that match a textual spec like <foo or <foo>bar """
-    fn text_range(fieldnames: &[&str], rng: &str) -> Result<Vec<usize>> {
-        let mut ret = Vec::new();
-        lazy_static! {
-            static ref RE1: Regex = Regex::new("^([<>]=?)([^<>]+)$").unwrap();
-            static ref RE2: Regex = Regex::new("^([<>]=?)([^<>]+)([<>]=?)([^<>]+)$").unwrap();
-        }
-
-        if let Some(caps) = RE1.captures(rng) {
-            for f in fieldnames.iter().enumerate() {
-                if Self::do_compare(
-                    f.1,
-                    caps.get(2).unwrap().as_str(),
-                    caps.get(1).unwrap().as_str(),
-                )? {
-                    ret.push(f.0);
-                }
-            }
-            return Ok(ret);
-        }
-
-        if let Some(caps) = RE2.captures(rng) {
-            for f in fieldnames.iter().enumerate() {
-                if Self::do_compare2(
-                    f.1,
-                    caps.get(2).unwrap().as_str(),
-                    caps.get(1).unwrap().as_str(),
-                    caps.get(4).unwrap().as_str(),
-                    caps.get(3).unwrap().as_str(),
-                )? {
-                    ret.push(f.0);
-                }
-            }
-            return Ok(ret);
-        }
-
-        err!("Malformed Range {}", rng)
-    }
-
     fn to_outcols(data: &[usize], name: &str) -> Vec<OutCol> {
         let mut ret = Vec::with_capacity(data.len());
         for x in data {
@@ -734,7 +732,7 @@ impl ColumnSet {
     /// turn comma delimited list of ranges into list of possibly named column numbers
     pub fn ranges(fieldnames: &[&str], rng: &str) -> Result<Vec<OutCol>> {
         let mut c = Self::new();
-        c.add_yes(rng);
+        c.add_yes(rng)?;
         c.lookup(fieldnames)?;
         Ok(c.get_cols_full())
     }
@@ -748,20 +746,13 @@ impl ColumnSet {
             Some(x) => x,
         };
 
-        let ch = range.chars().next().unwrap();
-        if ch == '<' || ch == '>' || ch == '=' {
+        let ch = range.first();
+        if ch == '(' {
             return Ok(Self::to_outcols(
-                &Self::text_range(fieldnames, range)?,
+                &Self::match_range(fieldnames, &range[1..range.len() - 1])?,
                 name,
             ));
         }
-        if range.contains('*') || rng.contains('?') {
-            return Ok(Self::to_outcols(
-                &Self::glob_range(fieldnames, range)?,
-                name,
-            ));
-        }
-
         let mut parts: Vec<&str> = range.split('-').collect();
         if parts.len() > 2 {
             return err!("Malformed Range {}", rng);
@@ -1034,7 +1025,7 @@ impl ColumnSet {
     /// ```
     pub fn lookup_cols(spec: &str, names: &[&str]) -> Result<Vec<usize>> {
         let mut s = Self::new();
-        s.add_yes(spec);
+        s.add_yes(spec)?;
         s.lookup(names)?;
         Ok(s.get_cols_num())
     }
@@ -1042,7 +1033,7 @@ impl ColumnSet {
     /// Shorthand to look up some columns, with names
     pub fn lookup_cols_full(spec: &str, names: &[&str]) -> Result<Vec<OutCol>> {
         let mut s = Self::new();
-        s.add_yes(spec);
+        s.add_yes(spec)?;
         s.lookup(names)?;
         Ok(s.get_cols_full())
     }
@@ -1058,7 +1049,7 @@ impl ColumnSet {
     /// ```
     pub fn lookup1(spec: &str, names: &[&str]) -> Result<usize> {
         let mut s = Self::new();
-        s.add_yes(spec);
+        s.add_yes(spec)?;
         s.lookup(names)?;
         if s.get_cols().len() != 1 {
             return err!(
@@ -1112,7 +1103,7 @@ impl ColumnClump {
         }
         let parts = parts.unwrap();
         let mut g = ColumnSet::new();
-        g.add_yes(parts.1);
+        g.add_yes(parts.1)?;
         let cols = Box::new(ReaderColumns::new(g));
         Ok(Self {
             cols,
@@ -1258,11 +1249,11 @@ pub struct NamedCol {
 
 impl fmt::Display for NamedCol {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(f, "Column {}", self.num)?;
+        write!(f, "Column {}", self.num + 1)?;
         if !self.name.is_empty() {
-            write!(f, " from {}", self.name)
+            write!(f, " ({})", self.name)
         } else if self.from_end != 0 {
-            write!(f, " from +{}", self.from_end)
+            write!(f, " (+{})", self.from_end)
         } else {
             Ok(())
         }
@@ -1306,44 +1297,30 @@ impl NamedCol {
         self.num = 0;
         self.name.clear();
         if spec.is_empty() {
-            return Ok(spec);
+            return err!("Empty string found where column name expected");
         }
         let mut ch = spec.first();
         let was_plus = ch == '+';
         if was_plus {
-            spec = spec.skip_first();
-            ch = spec.first();
+            ch = spec.take_first();
         }
-        if ch.is_ascii_digit() {
-            let mut pos: usize = 0;
-            for x in spec.chars() {
-                if x.is_ascii_digit() {
-                    self.num = self.num * 10 + (x as usize) - ('0' as usize);
-                    pos += 1;
-                } else {
-                    break;
-                }
-            }
+        if ch.is_ascii_digit() && ch != '0' {
+            let (a, b) = spec.to_usize();
+            self.num = a;
+            spec = b;
             if was_plus {
                 self.from_end = self.num;
             } else {
                 self.num -= 1;
             }
-            Ok(&spec[pos..])
+            Ok(spec)
         } else if ch.is_alphabetic() {
             if was_plus {
                 return err!("'+' must be follwed by a number : {}", orig_spec);
             }
-            let mut pos: usize = 0;
+            let pos = get_col_name(spec);
             self.name.clear();
-            for x in spec.chars() {
-                if x.is_alphanumeric() || x == '_' {
-                    self.name.push(x);
-                    pos += 1;
-                } else {
-                    break;
-                }
-            }
+            self.name += &spec[0..pos];
             Ok(&spec[pos..])
         } else {
             err!("Bad parse of compare spec for {}", spec)
@@ -1351,6 +1328,28 @@ impl NamedCol {
     }
 }
 
+/// return number of bytes used by the column name
+/// zero means no name present, return < str.len() if there's extra stuff
+pub fn get_col_name(spec: &str) -> usize {
+    if spec.is_empty() {
+        return 0;
+    }
+    let mut sp = spec;
+    let mut ch = sp.take_first();
+    if !ch.is_alphabetic() {
+        return 0;
+    }
+    while !sp.is_empty() {
+        ch = sp.first();
+        if ch.is_alphanumeric() || ch == '_' {
+            sp = sp.skip_first();
+        } else {
+            break;
+        }
+    }
+    spec.len() - sp.len()
+}
+
 #[derive(Debug, Clone, Default)]
 struct CompositeColumnPart {
     prefix: String,
@@ -1485,7 +1484,7 @@ mod tests {
             OutCol::from_num(4),
         ];
         assert_eq!(
-            ColumnSet::range(&f, "<p")?,
+            ColumnSet::range(&f, "(range,<p)")?,
             [OutCol::from_num(1), OutCol::from_num(4)]
         );
         assert_eq!(ColumnSet::range(&f, "2-+2")?, res[1..=3]);
@@ -1512,28 +1511,15 @@ mod tests {
     }
 
     #[test]
-    fn text_range() -> Result<()> {
-        let f: [&str; 5] = ["zero", "one", "two", "three", "four"];
-        assert_eq!(ColumnSet::text_range(&f, "<p")?, [1, 4]);
-        assert_eq!(ColumnSet::text_range(&f, "<p>g")?, [1]);
-        assert_eq!(ColumnSet::text_range(&f, ">=g<=p")?, [1]);
-        assert_err!(ColumnSet::text_range(&f, "><"), Err(Error::Error(_)));
-        return Ok(());
-    }
-    #[test]
-    fn do_compare() -> Result<()> {
-        assert_eq!(ColumnSet::do_compare("aaa", "bbb", "<")?, true);
-        assert_eq!(ColumnSet::do_compare("aaa", "bbb", "<=")?, true);
-        assert_eq!(ColumnSet::do_compare("aaa", "bbb", ">")?, false);
-        assert_eq!(ColumnSet::do_compare("aaa", "bbb", ">=")?, false);
-        assert_eq!(ColumnSet::do_compare("aaa", "aaa", "<")?, false);
-        assert_eq!(ColumnSet::do_compare("aaa", "aaa", "<=")?, true);
-        assert_eq!(ColumnSet::do_compare("aaa", "aaa", ">")?, false);
-        assert_eq!(ColumnSet::do_compare("aaa", "aaa", ">=")?, true);
-        assert_err!(
-            ColumnSet::do_compare("aaa", "aaa", "><"),
-            Err(Error::Error(_))
-        );
-        return Ok(());
+    fn do_get_col_name() {
+        assert_eq!(get_col_name(""), 0);
+        assert_eq!(get_col_name("..."), 0);
+        assert_eq!(get_col_name("_aaa"), 0);
+        assert_eq!(get_col_name("1aaa"), 0);
+        assert_eq!(get_col_name("abc"), 3);
+        assert_eq!(get_col_name("abc,"), 3);
+        assert_eq!(get_col_name("abc,aaa"), 3);
+        assert_eq!(get_col_name("a_b_c"), 5);
+        assert_eq!(get_col_name("a_ñ_c"), 6);
     }
 }
